
#include "dicts.h"
#include "eb123.h"
#include "ebgstsrc.h"
#include "history.h"
#include "mainwnd.h"
#include "render.h"
#include "textview.h"
#include "toolbar.h"

gboolean video_save_mpeg(RESULT *res, gchar *file)
{
    char binary_data[EB_SIZE_PAGE];
    guint argv[4];
    EB_Error_Code error_code;
    ssize_t read_length;
    FILE *fp;

    BOOK_INFO *binfo = dicts_find_binfo(res->book);
    if(!binfo)
	return FALSE;

    error_code = eb_set_subbook(binfo->book, binfo->subbook_no);
    if(error_code != EB_SUCCESS)
    {
	LOG(LOG_CRITICAL, "Failed to set subbook %s, %d : %s", binfo->path, binfo->subbook_no, eb_error_message(error_code));
	return FALSE;
    }

    if((error_code = eb_decompose_movie_file_name(argv, res->filename)) != EB_SUCCESS)
	return FALSE;

    error_code = eb_set_binary_mpeg(res->book, argv);
    if(error_code != EB_SUCCESS)
    {
	LOG(LOG_CRITICAL, "Failed to set binary mpeg: %s", eb_error_message(error_code));
	return FALSE;
    }

    fp = fopen(file, "wb");
    if(fp == NULL)
    {
	LOG(LOG_CRITICAL, "Failed to open file: %s", file);
	return FALSE;
    }

    for(;;)
    {
	error_code = eb_read_binary(res->book, EB_SIZE_PAGE, binary_data, &read_length);
	if(error_code != EB_SUCCESS || read_length == 0)
	{
	    fclose(fp);
	    return FALSE;
	}
	fwrite(binary_data, read_length, 1, fp);
    }

    return TRUE;
}

void video_save_as(GtkWidget *w, gpointer data)
{       
    static gchar *path = NULL;
    path = app_browse_disk(_("Save Video As"), GTK_WINDOW(mainwnd.wnd), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_SAVE_AS, path);
    if(path)
    {
        RESULT *res = (RESULT*)data;
        if(video_save_mpeg(res, path) != EB_SUCCESS)
            LOG(LOG_WARNING, _("Failed to save video"));
    }

}

gboolean video_context_menu(GtkWidget *w, GdkEventButton *event, gpointer user_data)
{
    if(event->button != 3) return FALSE;
    GtkWidget *menu = gtk_menu_new(), *item;
    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE_AS, NULL);
    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(video_save_as), user_data);
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
    gtk_widget_show_all(menu);
    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
    return TRUE;
}

#ifdef ENABLE_GSTREAMER

void video_play(GtkWidget *w, gpointer data)
{
    GMainLoop *loop = g_main_loop_new (NULL, FALSE);
    GstElement *src, *dec, *sink, *pipeline = gst_pipeline_new ("pipeline");
    GstBus *bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline));
    RESULT *res = (RESULT*)data;

    gst_bus_add_watch(bus, eb_gst_bus_callback, loop);
    gst_object_unref(bus);

    src = g_object_new(EB_GST_TYPE_SRC, NULL);
    EB_GST_SRC(src)->res = res;
    dec = gst_element_factory_make("mpeg2dec", "decoder");
    sink = gst_element_factory_make("xvimagesink", "videosink"); 

    gst_bin_add_many(GST_BIN(pipeline), src, dec, sink, NULL);
    gst_element_link_many(src, dec, sink, NULL);

    gst_element_set_state(pipeline, GST_STATE_PLAYING);
    g_main_loop_run(loop);

    gst_element_set_state(pipeline, GST_STATE_NULL);
    gst_object_unref(GST_OBJECT(pipeline));
}

gboolean video_render()
{
    GtkTextIter iter1, iter2;
    if(render_get_last_link(&iter1, &iter2))
    {
        gchar *title = gtk_text_buffer_get_text(render.buffer, &iter1, &iter2, FALSE);
        gtk_text_buffer_delete(render.buffer, &iter1, &iter2);
        gtk_text_buffer_get_end_iter(render.buffer, &iter1);
        gtk_text_buffer_insert(render.buffer, &iter1, "\n", -1);
        gtk_text_buffer_get_end_iter(render.buffer, &iter1);
        GtkTextChildAnchor *anchor = gtk_text_buffer_create_child_anchor(render.buffer, &iter1);
        GtkWidget *play = gtk_button_new();
	GtkWidget *image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY, GTK_ICON_SIZE_SMALL_TOOLBAR);
	gtk_container_add(GTK_CONTAINER(play), image);
	g_signal_connect(G_OBJECT(play), "pressed", G_CALLBACK(video_play), render.link);
	g_signal_connect(G_OBJECT(play), "button-press-event", G_CALLBACK(video_context_menu), render.link);
	gtk_widget_set_tooltip_text(play, _("[Video]"));
        g_free(title);
        gtk_text_view_add_child_at_anchor(render.view, play, anchor);
        gtk_text_buffer_get_end_iter(render.buffer, &iter2);
        gtk_text_buffer_insert(render.buffer, &iter2, "\n", -1);
        g_object_set_data_full(G_OBJECT(play), "video", render.link, result_free);
    }
    return TRUE;
}

#else

gboolean video_render()
{
    return FALSE;
}

#endif

